@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2020 Ian Ottoway <ian@ottoway.dev>
+# Copyright (c) 2021 Ian Ottoway <ian@ottoway.dev>
 # Copyright (c) 2014 Agostino Ruscito <ruscito@gmail.com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -22,50 +22,78 @@
 # SOFTWARE.
 #
 
-__all__ = ['EnumMap', ]
+__all__ = [
+    "EnumMap",
+]
+
+
+def _default_value_key(value):
+    return value
 
 
 class MapMeta(type):
-
     def __new__(cls, name, bases, classdict):
         enumcls = super().__new__(cls, name, bases, classdict)
 
         # get all non-private attributes
         members = {
-            key: value for key, value in classdict.items()
-            if not key.startswith('_') and not isinstance(value, (classmethod, staticmethod))
+            key: value
+            for key, value in classdict.items()
+            if not key.startswith("_")
+            and not isinstance(value, (classmethod, staticmethod))
+        }
+        # also add uppercase keys for each member (if they're not already lowercase)
+        lower_members = {
+            key.lower(): value
+            for key, value in members.items()
+            if key.lower() not in members
         }
 
-        # also add uppercase keys for each member (if they're not already lowercase)
-        lower_members = {key.lower(): value for key, value in members.items() if key.lower() not in members}
-
-        # invert members to a value->key dict
-        value_map = {value: key.lower() for key, value in members.items()}
+        if enumcls.__dict__.get("_bidirectional_", True):
+            # invert members to a value->key dict
+            _value_key = enumcls.__dict__.get("_value_key_", _default_value_key)
+            value_map = {
+                _value_key(value): key.lower() for key, value in members.items()
+            }
+        else:
+            value_map = {}
 
         # merge 3 previous dicts to get member lookup dict
         enumcls._members_ = {**members, **lower_members, **value_map}
+        enumcls._attributes = list(members)
 
         # lookup by value only return CAPS keys if attribute set
-        _only_caps = enumcls.__dict__.get('_return_caps_only_')
+        _only_caps = enumcls.__dict__.get("_return_caps_only_")
         enumcls._return_caps_only_ = _only_caps
 
         return enumcls
 
-    def __getitem__(self, item):
-        val = self._members_.__getitem__(item.lower() if isinstance(item, str) else item)
-        if self._return_caps_only_ and isinstance(val, str):
+    def __getitem__(cls, item):
+        val = cls._members_.__getitem__(_key(item))
+        if cls._return_caps_only_ and isinstance(val, str):
             val = val.upper()
         return val
 
     def get(cls, item, default=None):
-        val = cls._members_.get(item.lower() if isinstance(item, str) else item, default)
+
+        val = cls._members_.get(_key(item), default)
 
         if cls._return_caps_only_ and isinstance(val, str):
             val = val.upper()
         return val
 
-    def __contains__(self, item):
-        return self._members_.__contains__(item.lower() if isinstance(item, str) else item)
+    def __contains__(cls, item):
+        return cls._members_.__contains__(
+            item.lower() if isinstance(item, str) else item
+        )
+
+    @property
+    def attributes(cls):
+        return cls._attributes
+
+
+def _key(item):
+    return item.lower() if isinstance(item, str) else item
 
 
 class EnumMap(metaclass=MapMeta):
@@ -90,4 +118,5 @@ class EnumMap(metaclass=MapMeta):
     It's really just to provide dict-like item access with enum-like attributes.
 
     """
+
     ...
